@@ -1066,8 +1066,6 @@ class GenerationParameters(object):
 
     def __init__(self, random):
         self.__random = random
-        self.__zero_chance = None
-        self.__max_chance = None
         self.__pure_chance = None
         self.__alphabet = {}
 
@@ -1090,12 +1088,6 @@ class GenerationParameters(object):
         return self.__random.choice(alphabet)
 
     def __draw_without_alphabet(self, n):
-        if self.__random.random() <= self.zero_chance:
-            return hbytes(n)
-
-        if self.__random.random() <= self.max_chance:
-            return hbytes([255]) * n
-
         return uniform(self.__random, n)
 
     def alphabet(self, n_bytes):
@@ -1137,35 +1129,6 @@ class GenerationParameters(object):
         self.__alphabet[n_bytes] = result
         return result
 
-    @property
-    def max_chance(self):
-        """Returns a probability with which any given draw_bytes call should
-        be forced to be all 255 bytes. This is an important value because it
-        can make it more likely to push us into rare corners of the search
-        space, especially when biased_coin is being used."""
-        if self.__max_chance is None:
-            # We want to generate pure random examples every now and then. This
-            # is partly to offset too strong a bias to zero and partly because
-            # some user defined strategies may not play well with zero biasing.
-            self.__max_chance = self.__random.random() * 0.01
-        return self.__max_chance
-
-    @property
-    def zero_chance(self):
-        """Returns a probability with which any given draw_bytes call should
-        be forced to be all zero. This is an important value especially because
-        it will tend to force the test case to be smaller than it otherwise
-        would be."""
-        if self.__zero_chance is None:
-            # We want to generate pure random examples every now and then. This
-            # is partly to offset too strong a bias to zero and partly because
-            # some user defined strategies may not play well with zero biasing.
-            if self.__random.randrange(0, 10) == 0:
-                self.__zero_chance = 0.0
-            else:
-                self.__zero_chance = self.__random.random() * 0.5
-        return self.__zero_chance
-
     @property
     def pure_chance(self):
         """Returns a probability with which any given draw_bytes call should
