@@ -17,9 +17,11 @@
 
 from __future__ import absolute_import, division, print_function
 
+from collections import OrderedDict
+
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import OrderedDict
+from hypothesis.internal.conjecture.junkdrawer import LazySequenceCopy
 from hypothesis.internal.conjecture.utils import combine_labels
 from hypothesis.searchstrategy.strategies import (
     MappedSearchStrategy,
@@ -169,6 +171,40 @@ class UniqueListStrategy(ListStrategy):
         return result
 
 
+class UniqueSampledListStrategy(ListStrategy):
+    def __init__(self, elements, min_size, max_size, keys):
+        super(UniqueSampledListStrategy, self).__init__(elements, min_size, max_size)
+        self.keys = keys
+
+    def do_draw(self, data):
+        should_draw = cu.many(
+            data,
+            min_size=self.min_size,
+            max_size=self.max_size,
+            average_size=self.average_size,
+        )
+        seen_sets = tuple(set() for _ in self.keys)
+        result = []
+
+        remaining = LazySequenceCopy(self.element_strategy.elements)
+
+        while remaining and should_draw.more():
+            i = len(remaining) - 1
+            j = cu.integer_range(data, 0, i)
+            if j != i:
+                remaining[i], remaining[j] = remaining[j], remaining[i]
+            value = remaining.pop()
+
+            if all(key(value) not in seen for (key, seen) in zip(self.keys, seen_sets)):
+                for key, seen in zip(self.keys, seen_sets):
+                    seen.add(key(value))
+                result.append(value)
+            else:
+                should_draw.reject()
+        assert self.max_size >= len(result) >= self.min_size
+        return result
+
+
 class FixedKeysDictStrategy(MappedSearchStrategy):
     """A strategy which produces dicts with a fixed set of keys, given a
     strategy for each of their equivalent values.
@@ -199,3 +235,47 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
 
     def pack(self, value):
         return self.dict_type(zip(self.keys, value))
+
+
+class FixedAndOptionalKeysDictStrategy(SearchStrategy):
+    """A strategy which produces dicts with a fixed set of keys, given a
+    strategy for each of their equivalent values.
+
+    e.g. {'foo' : some_int_strategy} would generate dicts with the single
+    key 'foo' mapping to some integer.
+    """
+
+    def __init__(self, strategy_dict, optional):
+        self.required = strategy_dict
+        self.fixed = FixedKeysDictStrategy(strategy_dict)
+        self.optional = optional
+
+        if isinstance(self.optional, OrderedDict):
+            self.optional_keys = tuple(self.optional.keys())
+        else:
+            try:
+                self.optional_keys = tuple(sorted(self.optional.keys()))
+            except TypeError:
+                self.optional_keys = tuple(sorted(self.optional.keys(), key=repr))
+
+    def calc_is_empty(self, recur):
+        return recur(self.fixed)
+
+    def __repr__(self):
+        return "FixedAndOptionalKeysDictStrategy(%r, %r)" % (
+            self.required,
+            self.optional,
+        )
+
+    def do_draw(self, data):
+        result = data.draw(self.fixed)
+        remaining = [k for k in self.optional_keys if not self.optional[k].is_empty]
+        should_draw = cu.many(
+            data, min_size=0, max_size=len(remaining), average_size=len(remaining) / 2
+        )
+        while should_draw.more():
+            j = cu.integer_range(data, 0, len(remaining) - 1)
+            remaining[-1], remaining[j] = remaining[j], remaining[-1]
+            key = remaining.pop()
+            result[key] = data.draw(self.optional[key])
+        return result
