@@ -42,11 +42,12 @@ from hypothesis._settings import (
 )
 from hypothesis.control import current_build_context
 from hypothesis.core import given
-from hypothesis.errors import HypothesisException, InvalidArgument, InvalidDefinition
-from hypothesis.internal.compat import quiet_raise, string_types
-from hypothesis.internal.reflection import function_digest, nicerepr, proxies
+from hypothesis.errors import InvalidArgument, InvalidDefinition
+from hypothesis.internal.compat import hrange, quiet_raise, string_types
+from hypothesis.internal.reflection import function_digest, nicerepr, proxies, qualname
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
+from hypothesis.searchstrategy.featureflags import FeatureStrategy
 from hypothesis.searchstrategy.strategies import OneOfStrategy, SearchStrategy
 from hypothesis.vendor.pretty import CUnicodeIO, RepresentationPrinter
 
@@ -89,7 +90,17 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     @given(st.data())
     def run_state_machine(factory, data):
         machine = factory()
-        check_type(GenericStateMachine, machine, "state_machine_factory()")
+        if isinstance(machine, GenericStateMachine) and not isinstance(
+            machine, RuleBasedStateMachine
+        ):
+            note_deprecation(
+                "%s inherits from GenericStateMachine, which is deprecated.  Use a "
+                "RuleBasedStateMachine, or a test function with st.data(), instead."
+                % (type(machine).__name__,),
+                since="2019-05-29",
+            )
+        else:
+            check_type(RuleBasedStateMachine, machine, "state_machine_factory()")
         data.conjecture_data.hypothesis_runner = machine
 
         n_steps = settings.stateful_step_count
@@ -107,9 +118,16 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
 
             while should_continue.more():
                 value = data.conjecture_data.draw(machine.steps())
-                if print_steps:
-                    machine.print_step(value)
-                machine.execute_step(value)
+                # Assign 'result' here in case 'execute_step' fails below
+                result = multiple()
+                try:
+                    result = machine.execute_step(value)
+                finally:
+                    if print_steps:
+                        # 'result' is only used if the step has target bundles.
+                        # If it does, and the result is a 'MultipleResult',
+                        # then 'print_step' prints a multi-variable assignment.
+                        machine.print_step(value, result)
                 machine.check_invariants()
         finally:
             if print_steps:
@@ -127,6 +145,7 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     run_state_machine._hypothesis_internal_use_reproduce_failure = getattr(
         state_machine_factory, "_hypothesis_internal_use_reproduce_failure", None
     )
+    run_state_machine._hypothesis_internal_print_given_args = False
 
     run_state_machine(state_machine_factory)
 
@@ -150,25 +169,28 @@ class GenericStateMachineMeta(type):
 class GenericStateMachine(
     GenericStateMachineMeta("GenericStateMachine", (object,), {})  # type: ignore
 ):
-    """A GenericStateMachine is the basic entry point into Hypothesis's
-    approach to stateful testing.
+    """A GenericStateMachine is a deprecated approach to stateful testing.
 
-    The intent is for it to be subclassed to provide state machine descriptions
+    In earlier versions of Hypothesis, you would define ``steps``,
+    ``execute_step``, ``teardown``, and ``check_invariants`` methods;
+    and the engine would then run something like the following::
 
-    The way this is used is that Hypothesis will repeatedly execute something
-    that looks something like::
+        @given(st.data())
+        def test_the_stateful_thing(data):
+            x = MyStatemachineSubclass()
+            x.check_invariants()
+            try:
+                for _ in range(50):
+                    step = data.draw(x.steps())
+                    x.execute_step(step)
+                    x.check_invariants()
+            finally:
+                x.teardown()
 
-        x = MyStatemachineSubclass()
-        x.check_invariants()
-        try:
-            for _ in range(n_steps):
-                x.execute_step(x.steps().example())
-                x.check_invariants()
-        finally:
-            x.teardown()
-
-    And if this ever produces an error it will shrink it down to a small
-    sequence of example choices demonstrating that.
+    We now recommend using rule-based stateful testing instead wherever
+    possible.  If your test is better expressed in the above format than
+    as a rule-based state machine, we suggest "unrolling" your method
+    definitions into a simple test function with the above control flow.
     """
 
     def steps(self):
@@ -177,7 +199,10 @@ class GenericStateMachine(
         raise NotImplementedError(u"%r.steps()" % (self,))
 
     def execute_step(self, step):
-        """Execute a step that has been previously drawn from self.steps()"""
+        """Execute a step that has been previously drawn from self.steps()
+
+        Returns the result of the step execution.
+        """
         raise NotImplementedError(u"%r.execute_step()" % (self,))
 
     def print_start(self):
@@ -192,10 +217,10 @@ class GenericStateMachine(
         By default does nothing.
         """
 
-    def print_step(self, step):
+    def print_step(self, step, result):
         """Print a step to the current reporter.
 
-        This is called right before a step is executed.
+        This is called right after a step is executed.
         """
         self.step_count = getattr(self, u"step_count", 0) + 1
         report(u"Step #%d: %s" % (self.step_count, nicerepr(step)))
@@ -243,7 +268,7 @@ class GenericStateMachine(
 @attr.s()
 class Rule(object):
     targets = attr.ib()
-    function = attr.ib()
+    function = attr.ib(repr=qualname)
     arguments = attr.ib()
     precondition = attr.ib()
     bundles = attr.ib(init=False)
@@ -296,6 +321,23 @@ class Bundle(SearchStrategy):
         reference = data.draw(self.__reference_strategy)
         return machine.names_to_values[reference.name]
 
+    def __repr__(self):
+        consume = self.__reference_strategy.consume
+        if consume is False:
+            return "Bundle(name=%r)" % (self.name,)
+        return "Bundle(name=%r, consume=%r)" % (self.name, consume)
+
+    def calc_is_empty(self, recur):
+        # We assume that a bundle will grow over time
+        return False
+
+    def available(self, data):
+        # ``self_strategy`` is an instance of the ``st.runner()`` strategy.
+        # Hence drawing from it only returns the current state machine without
+        # modifying the underlying buffer.
+        machine = data.draw(self_strategy)
+        return bool(machine.bundle(self.name))
+
 
 class BundleConsumer(Bundle):
     def __init__(self, bundle):
@@ -403,6 +445,8 @@ def rule(targets=(), target=None, **kwargs):
     provided.
     """
     converted_targets = _convert_targets(targets, target)
+    for k, v in kwargs.items():
+        check_type(SearchStrategy, v, k)
 
     def accept(f):
         existing_rule = getattr(f, RULE_MARKER, None)
@@ -438,6 +482,8 @@ def initialize(targets=(), target=None, **kwargs):
     arbitrary order.
     """
     converted_targets = _convert_targets(targets, target)
+    for k, v in kwargs.items():
+        check_type(SearchStrategy, v, k)
 
     def accept(f):
         existing_rule = getattr(f, RULE_MARKER, None)
@@ -577,6 +623,10 @@ class RuleStrategy(SearchStrategy):
         self.machine = machine
         self.rules = list(machine.rules())
 
+        self.enabled_rules_strategy = st.shared(
+            FeatureStrategy(), key=("enabled rules", machine),
+        )
+
         # The order is a bit arbitrary. Primarily we're trying to group rules
         # that write to the same location together, and to put rules with no
         # target first as they have less effect on the structure. We order from
@@ -592,18 +642,37 @@ class RuleStrategy(SearchStrategy):
             )
         )
 
+    def __repr__(self):
+        return "%s(machine=%s({...}))" % (
+            self.__class__.__name__,
+            self.machine.__class__.__name__,
+        )
+
     def do_draw(self, data):
-        try:
-            rule = data.draw(st.sampled_from(self.rules).filter(self.is_valid))
-        except HypothesisException:
-            # FailedHealthCheck or UnsatisfiedAssumption depending on user settings.
+        if not any(self.is_valid(rule) for rule in self.rules):
             msg = u"No progress can be made from state %r" % (self.machine,)
             quiet_raise(InvalidDefinition(msg))
+
+        feature_flags = data.draw(self.enabled_rules_strategy)
+
+        # Note: The order of the filters here is actually quite important,
+        # because checking is_enabled makes choices, so increases the size of
+        # the choice sequence. This means that if we are in a case where many
+        # rules are invalid we will make a lot more choices if we ask if they
+        # are enabled before we ask if they are valid, so our test cases will
+        # be artificially large.
+        rule = data.draw(
+            st.sampled_from(self.rules)
+            .filter(self.is_valid)
+            .filter(lambda r: feature_flags.is_enabled(r.function.__name__))
+        )
+
         return (rule, data.draw(rule.arguments_strategy))
 
     def is_valid(self, rule):
         if rule.precondition and not rule.precondition(self.machine):
             return False
+
         for b in rule.bundles:
             bundle = self.machine.bundle(b.name)
             if not bundle:
@@ -659,6 +728,11 @@ class RuleBasedStateMachine(GenericStateMachine):
     def upcoming_name(self):
         return u"v%d" % (self.name_counter,)
 
+    def last_names(self, n):
+        assert self.name_counter > n
+        count = self.name_counter
+        return [u"v%d" % (i,) for i in hrange(count - n, count)]
+
     def new_name(self):
         result = self.upcoming_name()
         self.name_counter += 1
@@ -674,7 +748,7 @@ class RuleBasedStateMachine(GenericStateMachine):
         except KeyError:
             pass
 
-        for k, v in inspect.getmembers(cls):
+        for _, v in inspect.getmembers(cls):
             r = getattr(v, INITIALIZE_RULE_MARKER, None)
             if r is not None:
                 cls.define_initialize_rule(
@@ -690,7 +764,7 @@ class RuleBasedStateMachine(GenericStateMachine):
         except KeyError:
             pass
 
-        for k, v in inspect.getmembers(cls):
+        for _, v in inspect.getmembers(cls):
             r = getattr(v, RULE_MARKER, None)
             if r is not None:
                 cls.define_rule(r.targets, r.function, r.arguments, r.precondition)
@@ -705,7 +779,7 @@ class RuleBasedStateMachine(GenericStateMachine):
             pass
 
         target = []
-        for k, v in inspect.getmembers(cls):
+        for _, v in inspect.getmembers(cls):
             i = getattr(v, INVARIANT_MARKER, None)
             if i is not None:
                 target.append(i)
@@ -754,16 +828,27 @@ class RuleBasedStateMachine(GenericStateMachine):
     def print_end(self):
         report(u"state.teardown()")
 
-    def print_step(self, step):
+    def print_step(self, step, result):
         rule, data = step
         data_repr = {}
         for k, v in data.items():
             data_repr[k] = self.__pretty(v)
         self.step_count = getattr(self, u"step_count", 0) + 1
+        # If the step has target bundles, and the result is a MultipleResults
+        # then we want to assign to multiple variables.
+        if isinstance(result, MultipleResults):
+            n_output_vars = len(result.values)
+        else:
+            n_output_vars = 1
+        output_assignment = (
+            u"%s = " % (", ".join(self.last_names(n_output_vars)),)
+            if rule.targets and n_output_vars >= 1
+            else u""
+        )
         report(
             u"%sstate.%s(%s)"
             % (
-                u"%s = " % (self.upcoming_name(),) if rule.targets else u"",
+                output_assignment,
                 rule.function.__name__,
                 u", ".join(u"%s=%s" % kv for kv in data_repr.items()),
             )
@@ -793,6 +878,7 @@ class RuleBasedStateMachine(GenericStateMachine):
                 self._add_result_to_targets(rule.targets, result)
         if self._initialize_rules_to_run:
             self._initialize_rules_to_run.remove(rule)
+        return result
 
     def check_invariants(self):
         for invar in self.invariants():
